@@ -7,49 +7,163 @@ import gaussianprocessderivatives as gp
 from statsmodels.nonparametric.smoothers_lowess import lowess
 from scipy.optimize import minimize_scalar
 import pandas as pd
-import re
-import importlib.resources as import_files
 import om_code.sunder as sunder
 import om_code.omerrors as errors
 import om_code.omgenutils as gu
 
 
-def _read_dilution_data(fname):
-    d = import_files.read_text("om_code", fname)
-    res = np.array(re.split(r"\n|\t", d)[:-1]).astype(float)
-    od, dilfac = res[::2], res[1::2]
-    return od, dilfac
-
-
-def findODcorrection(wdirpath, ODfname, figs, odmatch, bd, gp_results):
+def findODcorrection(wdirpath, ODfname, exp, figs, odmatch):
     """
-    Determine a function to correct OD.
-
-    Use a Gaussian process to fit serial dilution data to correct for
-    non-linearities in the relationship between OD and cell density.
-
-    The data are either loaded from file ODfname or the default
-    data for haploid yeast growing in glucose are used.
+    Uses a Gaussian process to fit serial dilution data
+    to correct for non-linearities in the relationship between OD and cell
+    density. The data are either loaded from file ODfname or the default
+    data for haploid yeast growing in glucose are used (collected by
+    L Bandiera).
     """
     print("Fitting dilution data for OD correction for non-linearities")
-    if ODfname is not None:
+    if ODfname:
         try:
-            od_df = pd.read_csv(
-                str(wdirpath / ODfname), sep=None, engine="python", header=None
-            )
+            od_df = pd.read_csv(str(wdirpath / ODfname), sep=None, engine="python")
             print("Using", ODfname)
             od_data = od_df.to_numpy()
             od, dilfac = od_data[:, 0], od_data[:, 1]
         except (FileNotFoundError, OSError):
             raise errors.FileNotFound(str(wdirpath / ODfname))
     else:
-        print("Using default data")
-        fname = "dilution_data_xiao.tsv"
-        od, dilfac = _read_dilution_data(fname)
+        print("Using default data for haploid yeast in glucose")
+        # Lucia's data
+        od = np.array(
+            [
+                1.38192778,
+                1.15388333,
+                0.96450556,
+                0.78569444,
+                0.61685,
+                0.45291666,
+                0.33923888,
+                0.24501666,
+                0.18033889,
+                0.12463889,
+                0.08463889,
+                0.05783889,
+                0.04019444,
+                0.01776111,
+                0.01790556,
+                0.66888333,
+                0.47343889,
+                0.35310556,
+                0.25619444,
+                0.18245,
+                0.19291667,
+                0.12428333,
+                0.08719444,
+                0.05910555,
+                0.04203889,
+                0.02910556,
+                0.02035,
+                0.01392778,
+                0.00946111,
+                0.00651666,
+                1.06491665,
+                0.85690557,
+                0.71552777,
+                0.52646112,
+                0.39376111,
+                0.28846111,
+                0.20293889,
+                0.14296111,
+                0.10538333,
+                0.07017222,
+                0.04939444,
+                0.03397222,
+                0.02639445,
+                0.01780556,
+                0.01463889,
+                0.34130555,
+                0.21385,
+                0.15140556,
+                0.10612778,
+                0.07330556,
+                0.05147222,
+                0.03702778,
+                0.02599445,
+                0.01739444,
+                0.01188333,
+                0.00956111,
+                0.00572778,
+                0.00491667,
+                0.00275,
+                0.00316111,
+            ]
+        )
+        dilfac = np.array(
+            [
+                1.0000000e00,
+                7.1968567e-01,
+                5.1794747e-01,
+                3.7275937e-01,
+                2.6826958e-01,
+                1.9306977e-01,
+                1.3894955e-01,
+                1.0000000e-01,
+                7.1968570e-02,
+                5.1794750e-02,
+                3.7275940e-02,
+                2.6826960e-02,
+                1.9306980e-02,
+                1.3894950e-02,
+                1.0000000e-02,
+                3.0484230e-01,
+                2.1939064e-01,
+                1.5789230e-01,
+                1.1363283e-01,
+                8.1779920e-02,
+                5.8855830e-02,
+                4.2357700e-02,
+                3.0484230e-02,
+                2.1939060e-02,
+                1.5789230e-02,
+                1.1363280e-02,
+                8.1779900e-03,
+                5.8855800e-03,
+                4.2357700e-03,
+                3.0484200e-03,
+                4.4830728e-01,
+                3.2264033e-01,
+                2.3219962e-01,
+                1.6711074e-01,
+                1.2026721e-01,
+                8.6554590e-02,
+                6.2292100e-02,
+                4.4830730e-02,
+                3.2264030e-02,
+                2.3219960e-02,
+                1.6711070e-02,
+                1.2026720e-02,
+                8.6554600e-03,
+                6.2292100e-03,
+                4.4830700e-03,
+                7.7885310e-02,
+                5.6052940e-02,
+                4.0340500e-02,
+                2.9032480e-02,
+                2.0894260e-02,
+                1.5037300e-02,
+                1.0822130e-02,
+                7.7885300e-03,
+                5.6052900e-03,
+                4.0340500e-03,
+                2.9032500e-03,
+                2.0894300e-03,
+                1.5037300e-03,
+                1.0822100e-03,
+                7.7885000e-04,
+            ]
+        )
     # process data
     dilfac = dilfac[np.argsort(od)]
     od = np.sort(od)
-    if odmatch is not None:
+    if odmatch:
         # rescale so that OD and dilfac match at a particular OD
         # compares better with Warringer & Blomberg, Yeast 2003,
         # and rescaled OD is larger
@@ -57,16 +171,9 @@ def findODcorrection(wdirpath, ODfname, figs, odmatch, bd, gp_results):
         y = dilfac / dilfacmatch * odmatch
     else:
         y = dilfac
-    # set up Gaussian process
-    bds = {0: (-4, 4), 1: (-4, 4), 2: (-3, 1)}
-    # find bounds
-    if bd is not None:
-        bds = gu.mergedicts(original=bds, update=bd)
-    gc = gp.maternGP(bds, od, y)
     # run Gaussian process
+    gc = gp.sqexplinGP({0: (-4, 2), 1: (-3, 1), 2: (-6, 1), 3: (-6, 1)}, od, y)
     gc.findhyperparameters(noruns=5, exitearly=True, quiet=True)
-    if gp_results:
-        gc.results()
     gc.predict(od)
     if figs:
         plt.figure()
@@ -87,10 +194,8 @@ def findODcorrection(wdirpath, ODfname, figs, odmatch, bd, gp_results):
 
 def performmediacorrection(r_df, dtype, exp, condition, figs, commonmedia, frac):
     """
-    Correct data of type dtype for any signal from the media.
-
-    Use lowess to smooth over time the media data from the Null
-    wells and subtract the smoothed values from the data.
+    Uses lowess to smooth the media data from Null
+    wells over time and subtracts the smoothed values from the data.
     """
     # find data for correction with condition equal to commonmedia
     df = r_df.query(
@@ -159,14 +264,11 @@ def correctauto1(
     strainexcludes,
 ):
     """
-    Correct autofluorescence for measurements with emissions at one wavelength.
-
-    Corrects for autofluorescence for data with emissions measured at one
-    wavelength using the fluorescence of the reference strain
-    interpolated to the OD of the tagged strain.
-
-    This method in principle corrects too for the fluorescence of the medium,
-    although running correctmedia is still recommended.
+    Corrects for autofluorescence for experiments with measured
+    emissions at one wavelength using the fluorescence of the reference strain
+    interpolated to the OD of the tagged strain. This in principle corrects too
+    for the fluorescence of the medium, although running correctmedia is still
+    preferred.
     """
     print("Correcting autofluorescence using", f[0])
     for e in sunder.getexps(self, experiments, experimentincludes, experimentexcludes):
@@ -270,9 +372,8 @@ def correctauto1(
 
 def processref1(self, f, refstrain, figs, experiment, condition):
     """
-    Process reference strain for data with one fluorescence measurement.
-
-    Use lowess to smooth the fluorescence of the reference
+    Processes reference strain for data with one fluorescence
+    measurement. Uses lowess to smooth the fluorescence of the reference
     strain as a function of OD.
 
     Parameters
@@ -363,10 +464,8 @@ def correctauto2(
     strainexcludes,
 ):
     """
-    Correct autofluorescence for measurements with two emission wavelengths.
-
-    Corrects for autofluorescence using spectral unmixing for data with
-    measured emissions at two wavelengths.
+    Corrects for autofluorescence using spectral unmixing
+    for experiments with measured emissions at two wavelengths.
 
     References
     ----------
@@ -450,13 +549,10 @@ def correctauto2(
 
 def processref2(self, f, refstrain, figs, experiment, condition):
     """
-    Process reference strain for spectral unmixing.
-
-    Requires data with two fluorescence measurements.
-
-    Use lowess to smooth the ratio of emitted fluorescence measurements
-    so that the reference strain's data is corrected to zero as best
-    as possible.
+    Processes reference strain data for spectral unmixing
+    (for experiments with two fluorescence measurements). Uses lowess to
+    smooth the ratio of emitted fluorescence measurements so that the
+    reference strain's data is corrected to zero as best as possible.
 
     Parameters
     ----------
@@ -566,7 +662,9 @@ def processref2(self, f, refstrain, figs, experiment, condition):
 
 
 def applyautoflcorrection(self, ra, f0data, f1data):
-    """Correct for autofluorescence returning an array of replicates."""
+    """
+    Corrects for autofluorescence returning an array of replicates.
+    """
     nr = f0data.shape[1]
     raa = np.reshape(np.tile(ra, nr), (np.size(ra), nr), order="F")
     return (raa * f0data - f1data) / (raa - self._gamma * np.ones(np.shape(raa)))
@@ -576,7 +674,9 @@ def applyautoflcorrection(self, ra, f0data, f1data):
 
 
 def nanstdzeros2nan(a, axis=None):
-    """Like nanstd but setting zeros to nan."""
+    """
+    nanstd but setting zeros to nan
+    """
     err = np.nanstd(a, axis)
     err[err == 0] = np.nan
     return err
