@@ -18,7 +18,7 @@ import om_code.omstats as omstats
 import om_code.admin as admin
 import om_code.clogger as clogger
 
-version = "0.9.6"
+version = "0.9.45"
 
 plt.rcParams["figure.max_open_warning"] = 0
 sns.set()
@@ -26,17 +26,15 @@ sns.set()
 
 class platereader:
     """
-    To analyse plate-reader data.
-
-    Data are corrected for autofluorescence and growth rates found using
-    a Gaussian process.
+    for analyzing plate-reader data, correcting for autofluorescence, and
+    determining growth rates.
 
     All data is stored used Panda's dataframes and plotted using Seaborn.
 
     Three dataframes are created. If p is an instance of the platereader class,
     then p.r contains the raw data for each well in the plate; p.s contains the
     processed time-series using the data from all relevant wells; and p.sc
-    contains any summary statistics, such as 'max gr'.
+    constains any summary statistics, such as 'max gr'.
 
     For time series sampled from a Gaussian process, the mean is used as the
     statistics and errors are estimated by the standard deviation.
@@ -89,7 +87,9 @@ class platereader:
     >>> p.log
 
     See also
+
         http://swainlab.bio.ed.ac.uk/software/omniplate/index.html
+
     for a tutorial, which can be opened directly using
 
     >>> p.webhelp()
@@ -133,6 +133,10 @@ class platereader:
         asheetnumbers: integer or list of integers, optional
             The relevant sheets of the corresponding Excel files for the
             annotation.
+        ODfname: string, optional
+            The name of the file with the dilution data used to correct OD for
+            its non-linear dependence on numbers of cells. If unspecified, data
+            for haploid budding yeast growing in glucose is used.
         info: boolean
             If True (default), display summary information on the data once
             loaded.
@@ -145,6 +149,8 @@ class platereader:
         self.wdirpath = Path(wdir)
 
         # enable logging
+        # self._initialiselogging()
+        # self._logmethod(self.logger)
         self.logger, self.logstream = clogger.initlog(version)
 
         if True:
@@ -156,6 +162,8 @@ class platereader:
         self.progress = {
             "ignoredwells": {},
             "negativevalues": {},
+            "ODfname": {},
+            "gc": {},
         }
         self.allexperiments = []
         self.allconditions = {}
@@ -174,13 +182,13 @@ class platereader:
                 platereadertype,
                 dsheetnumbers,
                 asheetnumbers,
+                ODfname,
                 info,
             )
 
     ###
 
     def __repr__(self):
-        """Represent as a string."""
         repstr = "{} v{}: ".format(self.__class__.__name__, self.__version__)
         for e in self.allexperiments:
             repstr += e + " ; "
@@ -193,15 +201,15 @@ class platereader:
     def ls(self):
         """
         List all files in the working directory.
-
         A dictionary of available datasets is created as a shortcut.
 
         Example
         -------
         >>> p.ls
-        >>> p.files
-        >>> p.load(p.files[0], p.files[1])
+        >>> p.ds
+        >>> p.load(p.ds[0], p.ds[1])
         """
+        self.ds = {}
         print("Working directory is", str(self.wdirpath.resolve()))
         print("Files available are:", "\n---")
         files = []
@@ -216,7 +224,7 @@ class platereader:
                 print(f.stem + f.suffix)
                 files.append(f.stem + f.suffix)
         print()
-        self.files = {i: f for i, f in enumerate(sorted(files))}
+        self.ds = {i: f for i, f in enumerate(sorted(files))}
 
     ###
 
@@ -246,13 +254,12 @@ class platereader:
         platereadertype="Tecan",
         dsheetnumbers=False,
         asheetnumbers=False,
+        ODfname=None,
         info=True,
     ):
         """
-        Load raw data files.
-
-        Two files are required: one generated by the plate reader and the other
-        the corresponding annotation files - both assumed to be xlsx.
+        Loads raw data files generated by the plate reader and the
+        corresponding annotation files.
 
         Parameters
         ----------
@@ -270,6 +277,10 @@ class platereader:
         asheetnumbers: integer or list of integers, optional
             The relevant sheets of the corresponding Excel files for the
             annotation.
+        ODfname: string, optional
+            The name of the file with the dilution data used to correct OD for
+            its non-linear dependence on numbers of cells. If unspecified, data
+            for haploid budding yeast growing in glucose is used.
         info: boolean
             If True (default), display summary information on the data once
             loaded.
@@ -278,6 +289,8 @@ class platereader:
         -------
         >>> p.load('Data.xlsx', 'DataContents.xlsx')
         >>> p.load('Data.xlsx', 'DataContents.xlsx', info= False)
+        >>> p.load('Data.xlsx', 'DataContents.xlsx',
+        ...         ODfname= 'ODcorrection_Glucose_Diploid.txt')
         """
         dnames = gu.makelist(dnames)
         if not anames:
@@ -316,6 +329,17 @@ class platereader:
             # update progress dictionary
             admin.initialiseprogress(self, experiment)
 
+        # define ODfname in progress dictionary
+        if ODfname:
+            if isinstance(ODfname, str):
+                self.progress["ODfname"] = {exp: ODfname for exp in self.allexperiments}
+            else:
+                self.progress["ODfname"] = {
+                    exp: ODfname[i] for i, exp in enumerate(self.allexperiments)
+                }
+        else:
+            self.progress["ODfname"] = {exp: None for exp in self.allexperiments}
+
         # dataframe for summary stats and corrections
         alldfs = []
         # for exp in self.allexperiments:
@@ -364,7 +388,7 @@ class platereader:
     @property
     def info(self):
         """
-        Display conditions, strains, and datatypes.
+        Displays conditions, strains, and datatypes.
 
         Example
         -------
@@ -394,7 +418,7 @@ class platereader:
 
     def webhelp(self, browser=None):
         """
-        Open detailed examples of how to use in omniplate in a web browser.
+        Opens detailed examples of how to use in omniplate in a web browser.
 
         Parameters
         ----------
@@ -415,7 +439,9 @@ class platereader:
     @property
     def attributes(self):
         """
-        Print the attributes of the current instance.
+        Displays the names of the attributes of the current instance of
+        platereader and acts as a check to see what variables have been
+        calculated or determined.
 
         Example
         -------
@@ -440,23 +466,16 @@ class platereader:
     ###
 
     @clogger.log
-    def rename(self, translatedict, regex=True, **kwargs):
+    def rename(self, translatedict):
         """
-        Rename strains or conditions.
-
         Uses a dictionary to replace all occurrences of a strain or a condition
         with an alternative.
-
         Note that instances of self.progress will not be updated.
 
         Parameters
         ----------
         translatedict: dictionary
             A dictionary of old name - new name pairs
-        regex: boolean (optional)
-            Value of regex to pass to panda's replace.
-        kwargs: keyword arguments
-            Passed to panda's replace.
 
         Example
         -------
@@ -464,14 +483,15 @@ class platereader:
         """
         # replace in dataframes
         for df in [self.r, self.s, self.sc]:
-            df.replace(translatedict, inplace=True, regex=regex, **kwargs)
-        # replace in attributes - allstrains and allconditions
+            df.replace(translatedict, inplace=True)
+        # rename in attributes
+
+        def applydict(a):
+            return translatedict[a] if a in translatedict else a
+
         for e in self.allexperiments:
-            for listattr in [self.allconditions[e], self.allstrains[e]]:
-                for i, listitem in enumerate(listattr):
-                    for key in translatedict:
-                        if key in listitem:
-                            listattr[i] = listitem.replace(key, translatedict[key])
+            self.allconditions[e] = list(map(applydict, self.allconditions[e]))
+            self.allstrains[e] = list(map(applydict, self.allstrains[e]))
 
     ###
 
@@ -488,7 +508,8 @@ class platereader:
         strainexcludes=False,
     ):
         """
-        Display the current corrections to the data.
+        Displays the status of corrections made for the specified strains,
+        conditions, and experiments.
 
         Parameters
         ----------
@@ -554,8 +575,7 @@ class platereader:
     @clogger.log
     def addcolumn(self, newcolumnname, oldcolumn, newcolumnvalues):
         """
-        Add a new column to all dataframes by parsing an existing column.
-
+        Adds a new column to all dataframes by parsing an existing column.
         All possible entries for the new column are specified as strings and
         the entry in the new column will be whichever of these strings is
         present in the entry of the existing column.
@@ -600,13 +620,12 @@ class platereader:
         asstr=False,
     ):
         """
-        Add a new numeric column.
+        Adds a new numeric column by parsing the numbers from the entries of
+        an existing column.
+        It is best to run this command only after the basic analyses -
+        ignorewells, correctOD, and correctmedia - have been performed because
+        it changes the structure of the dataframes and may cause errors.
 
-        Parses the numbers from the entries of an existing column.
-
-        It is recommended to run only after the basic analyses - ignorewells,
-        correctOD, and correctmedia - have been performed because
-        it changes the structure of the dataframes.
 
         Parameters
         ----------
@@ -710,9 +729,8 @@ class platereader:
         strainexcludes=False,
     ):
         """
-        Apply func to a column in the s dataframe.
-
-        The results are stored in the sc dataframe.
+        Applies func to a column in the s dataframe and
+        stores the results in the sc dataframe.
 
         Parameters
         ----------
@@ -726,6 +744,7 @@ class platereader:
 
         Examples
         --------
+
         >>> p.add_to_sc(newcolumn= "max GFP", s_column= "GFP mean",
         ...             func= np.nanmax)
         >>> p.add_to_sc(newcolumn= "GFP lower quartile", s_column= "GFP mean",
@@ -749,7 +768,7 @@ class platereader:
             for c in cons:
                 for s in strs:
                     d = self.s.query(
-                        "experiment == @e and condition == @c and strain == @s"
+                        "experiment == @e and condition == @c and " "strain == @s"
                     )[s_column].values
                     res = np.asarray(func(d))
                     if res.size == 1:
@@ -784,17 +803,13 @@ class platereader:
         strainexcludes=False,
     ):
         """
-        Add a common variable to all time-dependent dataframes.
-
-        The common variable allows averaging across experiments
+        Adds to time-dependent dataframes a common variable whose values only
+        come from a fixed array so that they are from the same array for all
+        experiments. This common variable allows averaging across experiments
         and typically is time.
 
-        A common variable is added to time-dependent dataframes. This
-        variable's values only come from a fixed array so that they are
-        from the same array for all experiments.
-
         For example, the plate reader often does not perfectly increment time
-        between measurements and different experiments can have slightly
+        between measurements and different experients can have slightly
         different time points despite the plate reader having the same
         settings. These unique times prevent seaborn from taking averages.
 
@@ -939,7 +954,7 @@ class platereader:
 
     def contentsofwells(self, wlist):
         """
-        Display contents of wells.
+        Displays contents of wells
 
         Parameters
         ----------
@@ -976,7 +991,7 @@ class platereader:
         strainexcludes=False,
     ):
         """
-        Display wells for specified experiments, conditions, and strains.
+        Displays wells for specified experiments, conditions, and strains.
 
         Parameters
         ----------
@@ -1057,8 +1072,7 @@ class platereader:
         clearall=False,
     ):
         """
-        Ignore the wells specified in any future processing.
-
+        Allows wells to be ignored in any future processing.
         If called several times, the default behaviour is for any previously
         ignored wells not to be re-instated.
 
@@ -1150,9 +1164,8 @@ class platereader:
     @clogger.log
     def restricttime(self, tmin=None, tmax=None):
         """
-        Restrict the processed data to a range of time.
-
-        Points outside this time range are ignored.
+        Restrict the processed data to a range of time, ignoring points outside
+        this time range.
 
         Note that data in the .s dataframe outside the time range is lost.
         Exporting the dataframes before running restricttime is recommended.
@@ -1193,8 +1206,7 @@ class platereader:
         row=None,
         height=5,
         aspect=1,
-        xlim=None,
-        ylim=None,
+        ymin=None,
         figsize=False,
         returnfacetgrid=False,
         title=None,
@@ -1215,9 +1227,8 @@ class platereader:
         **kwargs,
     ):
         """
-        Plot from the underlying dataframes (chosen automatically).
-
-        Seaborn's relplot is used, which is described at
+        Plots from the underlying dataframes (chosen automatically) using
+        Seaborn's relplot, which is described at
         https://seaborn.pydata.org/generated/seaborn.relplot.html
 
         Parameters
@@ -1250,10 +1261,8 @@ class platereader:
         aspect: float, optional
             The aspect ratio of the individual panels in a multipanel plot.
             From Seaborn.
-        xlim: list of two floats, optional
-            The minimal and maximal x-value, such as [0, None]
-        ylim: list of two floats, optional
-            The minimal and maximal y-value, such as [0, None]
+        ymin: float, optional
+            The minimum y-value
         figsize: tuple, optional
             A tuple of (width, height) for the size of figure.
             Ignored if wells= True or plate= True.
@@ -1351,14 +1360,13 @@ class platereader:
                 kind,
                 col,
                 row,
-                xlim,
-                ylim,
+                ymin,
                 title,
                 messages,
                 **kwargs,
             )
         elif dfname == "s" or dfname == "r":
-            sfig = omplot.plot_rs(
+            omplot.plot_rs(
                 x,
                 y,
                 basedf,
@@ -1373,16 +1381,13 @@ class platereader:
                 row,
                 height,
                 aspect,
-                xlim,
-                ylim,
+                ymin,
                 title,
                 figsize,
                 sortby,
                 returnfacetgrid,
                 **kwargs,
             )
-            if returnfacetgrid:
-                return sfig
         elif dfname == "sc":
             omplot.plot_sc(
                 x,
@@ -1399,8 +1404,7 @@ class platereader:
                 row,
                 height,
                 aspect,
-                xlim,
-                ylim,
+                ymin,
                 figsize,
                 title,
                 sortby,
@@ -1413,9 +1417,8 @@ class platereader:
 
     def savefigs(self, fname=None, onefile=True):
         """
-        Save all current figures to PDF.
-
-        Either all figures save to one file or each to a separate one.
+        Saves all current figures to PDF, either to one file or each to a
+        separate file.
 
         Parameters
         ----------
@@ -1477,9 +1480,8 @@ class platereader:
         nonull=True,
     ):
         """
-        Obtain a subset of the data in a dataframe.
-
-        This data can be used plotted directly.
+        Obtain a subset of the data in a dataframe, which can be used plotting
+        directly.
 
         Parameters
         ---------
@@ -1554,9 +1556,6 @@ class platereader:
         self,
         figs=True,
         odmatch=0.3,
-        bd=None,
-        gp_results=False,
-        ODfname=None,
         experiments="all",
         experimentincludes=False,
         experimentexcludes=False,
@@ -1565,8 +1564,8 @@ class platereader:
         conditionexcludes=False,
     ):
         """
-        Correct for the non-linear relationship between OD and cell number.
-
+        Corrects OD data for the non-linear relationship between OD and cell
+        number.
         Requires a set of dilution data set, with the default being haploid
         yeast growing in glucose (collected by L Bandiera).
         An alternative can be loaded from a file - a txt file of two columns
@@ -1578,21 +1577,9 @@ class platereader:
         figs: boolean, optional
             If True, a plot of the fit to the dilution data is produced.
         odmatch: float, optional
-            If not None, then the corrected OD is rescaled to equal the
+            If non-zero, then the corrected OD is rescaled to equal the
             measured OD at this value. Only large ODs typically need to be
             corrected.
-        bd: dictionary, optional
-            Specifies the limits on the hyperparameters for the Gaussian
-            process.
-            For example, bd= {0: [-1, 4], 2: [2, 6]})
-            sets confines the first hyperparameter to be between 1e-1 and 1e^4
-            and confines the third hyperparmater between 1e2 and 1e6.
-        gp_results: boolean, optional
-            If True, show the results of fitting the Gaussian process
-        ODfname: string, optional
-            The name of the file with the dilution data used to correct OD for
-            its non-linear dependence on numbers of cells. If unspecified, data
-            for haploid budding yeast growing in glucose is used.
         experiments: string or list of strings
             The experiments to include.
         conditions: string or list of strings
@@ -1622,16 +1609,6 @@ class platereader:
             conditionexcludes,
             nomedia=False,
         )
-        # fit dilution data
-        gc = corrections.findODcorrection(
-            self.wdirpath,
-            ODfname,
-            figs=figs,
-            odmatch=odmatch,
-            bd=bd,
-            gp_results=gp_results,
-        )
-        # correct ODs
         for exp in exps:
             for c in cons:
                 if self.sc[(self.sc.experiment == exp) & (self.sc.condition == c)][
@@ -1639,20 +1616,39 @@ class platereader:
                 ].any():
                     print(exp, ": OD is already corrected for", c)
                 else:
+                    # fit dilution data
+                    if not self.progress["gc"][exp]:
+                        ODfname = (
+                            str(self.progress["ODfname"][exp])
+                            if self.progress["ODfname"][exp]
+                            else None
+                        )
+                        gc = corrections.findODcorrection(
+                            self.wdirpath,
+                            ODfname,
+                            exp,
+                            figs,
+                            odmatch,
+                        )
+                        self.progress["gc"][exp] = gc
+                        # copy gc to experiments with the same ODfname
+                        for e in self.allexperiments:
+                            if (
+                                self.progress["ODfname"][e]
+                                == self.progress["ODfname"][exp]
+                            ):
+                                self.progress["gc"][e] = gc
                     # correct all wells
-                    r_data = self.r.query("experiment == @exp and condition == @c")[
-                        "OD"
-                    ].to_numpy()
-                    gc.batchpredict(r_data)
-                    # leave small ODs unchanged
-                    new_r = gc.f
-                    if odmatch is not None:
-                        new_r[r_data < odmatch] = r_data[r_data < odmatch]
-                    # update r data frame
+                    gc.batchpredict(
+                        self.r.query("experiment == @exp and condition == @c")[
+                            "OD"
+                        ].to_numpy()
+                    )
+                    # update r dataframe
                     self.r.loc[
                         (self.r.experiment == exp) & (self.r.condition == c),
                         "OD",
-                    ] = new_r
+                    ] = gc.f
                     # flag corrections in summary stats dataframe
                     self.sc.loc[
                         (self.sc.experiment == exp) & (self.sc.condition == c),
@@ -1680,10 +1676,8 @@ class platereader:
         frac=0.33,
     ):
         """
-        Correct OD or fluorescence for that of the media.
-
-        Data from wells marked Null is used.
-
+        Corrects OD or fluorescence for the OD or fluorescence of the media
+        using data from wells marked Null.
         Uses lowess to smooth measurements of from all Null wells and subtracts
         this smoothed time series from the raw data.
 
@@ -1806,10 +1800,9 @@ class platereader:
         **kwargs,
     ):
         """
-        Estimate first and second time derivatives.
-
-        The derivatives are found, typically of OD, using a Gaussian process
-        (Swain et al., 2016), as well as corresponding summary statistics.
+        Calls fitderiv.py to estimate the first and second time-derivatives of,
+        typically, OD using a Gaussian process (Swain et al., 2016) and find
+        corresponding summary statistics.
 
         The derivatives are stored in the .s dataframe; summary statistics are
         stored in the .sc dataframe.
@@ -1969,7 +1962,7 @@ class platereader:
                         )
                         # add columns plus and minus err
                         df = omplot.augmentdf(df, dtype)[[dtype, "augtype", "time"]]
-                        piv_df = df.pivot(index="time", columns="augtype", values=dtype)
+                        piv_df = df.pivot("time", "augtype", dtype)
                         # convert to array for fitderiv
                         d = piv_df.values
                         t = piv_df.index.to_numpy()
@@ -1977,7 +1970,7 @@ class platereader:
                         if np.any(numberofnans):
                             print(f"\nWarning: {numberofnans} NaNs in data")
                     else:
-                        print(dtype, "not recognised for", figtitle)
+                        print(dtype, "not recognized for", figtitle)
                         return
                     # checks
                     if d.size == 0:
@@ -2085,11 +2078,10 @@ class platereader:
         plot=False,
     ):
         """
-        Average a time-dependent variable over all experiments.
+        Uses a Matern Gaussian process to average a time-dependent variable
+        over all experiments.
 
-        Uses a Matern Gaussian process.
-
-        An alternative and best first choice is addcommonvar.
+        An alternative and best first choice is to use addcommonvar.
 
         Parameters
         ----------
@@ -2174,10 +2166,8 @@ class platereader:
         strainexcludes=False,
     ):
         """
-        Correct fluorescence for autofluorescence.
-
-        The correction is made using the fluorescence of an untagged
-        reference strain.
+        Corrects fluorescence data for autofluorescence by comparing with the
+        fluorescence of an untagged reference strain.
 
         The reference strain is used to estimate the autofluoresence via either
         the method of Licthen et al., 2014, where measurements of fluoescence
@@ -2360,7 +2350,7 @@ class platereader:
     @property
     def log(self):
         """
-        Print a log of all methods called and their arguments.
+        Prints a log of all methods called and their arguments.
 
         Example
         -------
@@ -2398,9 +2388,7 @@ class platereader:
     @clogger.log
     def exportdf(self, commonname=False, type="tsv"):
         """
-        Export the dataframes.
-
-        The exported data may either be tab-delimited or csv or json files.
+        Exports the dataframes as either tab-delimited or csv or json files.
         Dataframes for the (processed) raw data, for summary data, and for
         summary statistics and corrections, as well as a log file, are
         exported.
@@ -2464,7 +2452,7 @@ class platereader:
                         "'.json', orient= 'split')"
                     )
                     print("Imported", commonname + "_" + df + ".json")
-                except FileNotFoundError:
+                except ValueError:
                     try:
                         # csv files
                         exec(
